#ifdef USE_PRAGMA_IDENT_HDR
#pragma ident "@(#)vm_version_sparc.hpp	1.32 07/05/05 17:04:33 JVM"
#endif
/*
 * Copyright 1997-2007 Sun Microsystems, Inc.  All Rights Reserved.
 * Copyright 2007-2008 Reservoir Labs, Inc.  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
 * CA 95054 USA or visit www.sun.com if you need additional information or
 * have any questions.
 *  
 */

class VM_Version : public Abstract_VM_Version {
public:
  // config register layouts.  These are all unions of a uint32_t
  // (in case anyone wants access to the register as a whole) and a bitfield.

  union ProcessorReg {
    uint32_t value;
    struct {
      uint32_t revision       : 8, // revision of same processor type
               processorID    : 8, // processor type
               companyID      : 8, // 
               companyOptions : 8; // company-specific flags
    } bits;
  };

  union ConfigReg {
    uint32_t value;
    struct {
      uint32_t K0  : 3, // kseg0 cache coherency alg
                   : 4, // 0
               MT  : 3, // MMU type. 1=standard TLB, 3=standard FMT
               AR  : 3, // architecture revision level
      // avoid conflict with AT register macro
               AT_ : 2, // architecture type
                        // 1 = MIPS64 with 32-bit addresses only
                        // 2 = MIPS64 with 32/64-bit addresses
               BE  : 1, // endianess: 0=little, 1=big
               BM  : 1, // burst mode: 0=incremental, 1=interleaved
               DID : 1, // dual issue disable: 0=enable, 1=disable
                   : 2, // 0
               ISD : 1, // instr sched disable: 0=enable, 1=disable
               SB  : 1, // simpleBE bus mode
                   : 3, // 0
               KU  : 3, // useg/kseg coherency alg., with FMT-based MMU
               K23 : 3, // kseg2, kseg 3 coherency alg.
               M   : 1; // 1= Config1 register is implemented
    } bits;
  };

  union ConfigReg1 {
    uint32_t value;
    struct {
      uint32_t FP_ : 1, // FPU implemented: 0=no, 1=yes
               EP  : 1, // EJTAG implemented: 0=no, 1=yes
               CA  : 1, // Code compression (MIPS16 ASE): 0=no, 1=yes
               WR  : 1, // Watch registers implemented: 0=no, 1=yes
               PC  : 1, // performance counter implemented: 0=no, 1=yes
               MD  : 1, // MDMX ASE implemented: 0=no, 1=yes
               C2  : 1, // Coprocessor 2 implemented: 0=no, 1=yes
               DA  : 3, // Dcache set associativity
                        // 0: direct mapped
                        // 1: 2-way
                        // 2: 3-way
                        // 3: 4-way
               DL  : 3, // Dcache line size
                        // 0: no Icache
                        // 4: 32 bytes
               DS  : 3, // Dcache sets per way
                        // 1: 128 sets
                        // 2: 256 sets
                        // 3: 512 sets
               IA  : 3, // Icache set associativity
                        // 0: direct mapped
                        // 1: 2-way
                        // 2: 3-way
                        // 3: 4-way
               IL  : 3, // Icache line size
                        // 0: no Icache
                        // 4: 32 bytes
               IS  : 3, // Icache sets per way
                        // 1: 128 sets
                        // 2: 256 sets
                        // 3: 512 sets
               MMU : 6, // number of entries in TLB minus 1.  0 = no TLB
                   : 1; // 0
    } bits;
  };

  struct ProcInfo {
    ProcessorReg processor_reg;
    ConfigReg    config_reg;
    ConfigReg1   config1_reg;
  };

protected:

  static ProcInfo _proc_info;

  static const char* _features_str;

  static ByteSize procinfo_proc_reg_offset() {
    return byte_offset_of(ProcInfo, processor_reg);
  }
  static ByteSize procinfo_config_reg_offset() {
    return byte_offset_of(ProcInfo, config_reg);
  }
  static ByteSize procinfo_config1_reg_offset() {
    return byte_offset_of(ProcInfo, config1_reg);
  }

  static bool config1_implemented() {
    return _proc_info.config_reg.bits.M;
  }

  friend class VM_Version_StubGenerator; // needs the above

  static void get_processor_features();

public:
  // Initialization
  static void initialize();

  // Asserts
  static void assert_is_initialized() {
    assert(_proc_info.processor_reg.value != 0, "VM_Version not initialized");
  }

  static const char* cpu_features()     { return _features_str; }

  // Arch / Feature identification

  static bool big_endian() {
    return _proc_info.config_reg.bits.BE;
  }

  static bool has_FPU() {
    return _proc_info.config1_reg.bits.FP_;
  }

  static int Dcache_set_associativity() {  // in "ways"
    return _proc_info.config1_reg.bits.DA + 1;
  }

  static int Icache_set_associativity() {  // in "ways"
    return _proc_info.config1_reg.bits.IA + 1;
  }

  static int Dcache_line_size() {  // in bytes
    return _proc_info.config1_reg.bits.DL * 4;
  }

  static int Icache_line_size() {  // in bytes
    return _proc_info.config1_reg.bits.IL * 4;
  }

  static int Dcache_sets_per_way() {
    return 1 << (_proc_info.config1_reg.bits.DS + 6);
  }

  static int Icache_sets_per_way() {
    return 1 << (_proc_info.config1_reg.bits.IS + 6);
  }

  static int MMU_size() {
    return _proc_info.config1_reg.bits.MMU + 1;
  }

  // There are more feature flags; add any others that seem useful
  // here.

};
